/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

//`define SUPPORT_SIO_COMATIBLE

//
// Module Name: SN74281
//   ver 001
// 
// Target Device:   xc3s200a-4ft256
// Product Version: ISE 12.4
// Design Goal:     Balanced
//
//                           ALU_WIDTH:   4         8
// Number of Slice Flip Flops:            4         8
// Number of 4 input LUTs:               79       115
// Number of occupied Slices:            40        60
// Total Number of 4 input LUTs:         79       116
// Number of bonded IOBs:                22        30

// Post-PAR Static Timing Report
// Maximum frequency(MHz):           144.238    97.466
//
module SN74382_ACC # (
	parameter ALU_WIDTH = 8
) (
		 input   C_IN       // Carry Input
		, input  [ALU_WIDTH-1:0] A     // DATA IN
		, output [ALU_WIDTH-1:0] F   // DATA OUT
		, output  C_OUT      // carry-look-ahead
		, output  OVR        // overflow

		, input   [2:0] S    // Function Select
		, input   [1:0] RS   // Reg. Select
		, input   RC         // Reg. Control
		, input   CP         // Clock 
`ifdef SUPPORT_SIO_COMATIBLE
		, inout   SIO0        // RI/LO
		, inout   SIO3        // LI/RO
`else
		, input   SI0        // RI/LO
		, output  SO0        // RI/LO
		, input   SI3        // LI/RO
		, output  SO3        // LI/RO
`endif
		);
	wire [ALU_WIDTH-1:0] B;

	alu_74382 # ( .ALU_WIDTH(ALU_WIDTH) ) ALU (
		  .C_IN(C_IN)     // Carry Input
		, .A(A[ALU_WIDTH-1:0])      // DATA IN
		, .B(B[ALU_WIDTH-1:0])      // DATA IN
		, .F(F[ALU_WIDTH-1:0])      // DATA OUT
		, .C_OUT(C_OUT)   // carry-look-ahead
		, .OVR(OVR)       // overflow
		, .S(S[2:0])      // Function Select
		);
	acc_74281 # ( .ALU_WIDTH(ALU_WIDTH)) ACC (
		  .B(B[ALU_WIDTH-1:0])      // DATA OUT
		, .F(F[ALU_WIDTH-1:0])      // DATA IN

		, .RS(RS[1:0])    // Reg. Select
		, .RC(RC)         // Reg. Control
		, .CP(CP)         // Clock 
		, .SI0(SI0)       // RI/LO
		, .SO0(SO0)       // RI/LO
		, .SI3(SI3)       // LI/RO
		, .SO3(SO3)       // LI/RO
		);

`ifdef SUPPORT_SIO_COMATIBLE
	assign SIO0 = (RS == 2) ? SO0 : "Z" ;
	assign SIO3 = (RS == 1) ? SO3 : "Z" ;

	assign SI0 = SIO0;
	assign SI3 = SIO3;
`endif

endmodule

