/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

`define SHRINKED_LUT

module alu_74382 # (
	parameter ALU_WIDTH = 4
) (
		  input   C_IN       // Carry Input
		, input  [ALU_WIDTH -1:0] A    // DATA IN
		, input  [ALU_WIDTH -1:0] B    // DATA IN
		, output [ALU_WIDTH -1:0] F    // DATA OUT
		, output  C_OUT      // carry-look-ahead
		, output  OVR        // overflow
		, input  [2:0] S   // Function Select

//		, input   CP       // Clock
		);


//`define ZZZ
//`define YYY
`ifdef ZZZ
    wire [ALU_WIDTH -2:0] minus1 = -1;
    wire [ALU_WIDTH:0] Rd_out =
		  (S == 1) ? B - A - C_IN
		: (S == 2) ? A - B - C_IN
		: (S == 3) ? A + B + C_IN
		: (S == 4) ? { 1'b0,  (A ^ B) } 
		: (S == 5) ? { 1'b0,  (A | B) }
		: (S == 6) ? { 1'b0,  (A & B) }
		: (S == 7) ? { 1'b0, minus1 , C_IN }
		: 0;

    assign F     = Rd_out[ALU_WIDTH -1:0];
    assign C_OUT = Rd_out[ALU_WIDTH];

`elsif YYY
    wire [ALU_WIDTH -2:0] minus1 = -1;
    wire [ALU_WIDTH -1:0] Rd_in = (S == 1) ? B : A;
    wire [ALU_WIDTH -1:0] Rr_in = (S == 1) ? A : B;

    wire [ALU_WIDTH:0] Rd_out = 
		 ((S == 1) | (S == 2)) ? Rd_in  - Rd_in - C_IN
		: (S == 3) ? Rd_in + Rr_in + C_IN
		: (S == 4) ? { 1'b0,  (Rd_in ^ Rr_in) } 
		: (S == 5) ? { 1'b0,  (Rd_in | Rr_in) }
		: (S == 6) ? { 1'b0,  (Rd_in & Rr_in) }
		: (S == 7) ? { 1'b0, minus1 , C_IN }
		: 0;

   assign OVR = (S == 3) ? (Rd_out[ALU_WIDTH -1] 
				& ~Rd_in[ALU_WIDTH -1] & ~Rr_in[ALU_WIDTH -1]
                	    | ~Rd_out[ALU_WIDTH -1] 
				&  Rd_in[ALU_WIDTH -1] &  Rr_in[ALU_WIDTH -1] )
	      : ( (S == 1) | (S == 2) )
			? (Rd_out[ALU_WIDTH -1]
				 & ~Rd_in[ALU_WIDTH -1] &  Rr_in[ALU_WIDTH -1]
               			| ~Rd_out[ALU_WIDTH -1]
				 &  Rd_in[ALU_WIDTH -1] & ~Rr_in[ALU_WIDTH -1] )
	      : 0 ;

    assign F     = Rd_out[ALU_WIDTH -1:0];
    assign C_OUT = Rd_out[ALU_WIDTH];
`else // using LUT
     // input { S , C_IN, A , B }
     // output { F , C_OUT , OVR }

    wire [ALU_WIDTH:0] CN;

    assign CN[0]  = C_IN;
    generate
       genvar i;
       for (i = 0; i < ALU_WIDTH; i=i+1)
           begin : gen_lut
		assign F[i]     = 
			  ({ S , CN[i], A[i] , B[i] } == 0) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 1) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 2) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 3) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 4) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 5) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 6) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 7) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 8) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 9) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 10) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 11) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 12) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 13) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 14) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 15) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 16) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 17) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 18) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 19) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 20) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 21) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 22) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 23) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 24) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 25) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 26) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 27) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 28) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 29) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 30) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 31) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 32) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 33) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 34) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 35) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 36) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 37) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 38) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 39) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 40) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 41) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 42) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 43) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 44) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 45) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 46) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 47) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 48) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 49) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 50) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 51) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 52) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 53) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 54) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 55) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 56) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 57) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 58) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 59) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 60) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 61) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 62) ? 1'b1
			: 1'b1;

		assign CN[i+1]  =
			  ({ S , CN[i], A[i] , B[i] } == 0) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 1) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 2) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 3) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 4) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 5) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 6) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 7) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 8) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 9) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 10) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 11) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 12) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 13) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 14) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 15) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 16) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 17) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 18) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 19) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 20) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 21) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 22) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 23) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 24) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 25) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 26) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 27) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 28) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 29) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 30) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 31) ? 1'b1
`ifdef SHRINKED_LUT
			: 1'b0;
`else
			: ({ S , CN[i], A[i] , B[i] } == 32) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 33) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 34) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 35) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 36) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 37) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 38) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 39) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 40) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 41) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 42) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 43) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 44) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 45) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 46) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 47) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 48) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 49) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 50) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 51) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 52) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 53) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 54) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 55) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 56) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 57) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 58) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 59) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 60) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 61) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 62) ? 1'b0
			: 1'b1;
`endif
		if (i == ALU_WIDTH -1 ) assign OVR =
			  ({ S , CN[i], A[i] , B[i] } == 0) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 1) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 2) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 3) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 4) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 5) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 6) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 7) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 8) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 9) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 10) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 11) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 12) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 13) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 14) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 15) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 16) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 17) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 18) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 19) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 20) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 21) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 22) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 23) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 24) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 25) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 26) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 27) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 28) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 29) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 30) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 31) ? 1'b0
`ifdef SHRINKED_LUT
			: 1'b0;
`else
			: ({ S , CN[i], A[i] , B[i] } == 32) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 33) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 34) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 35) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 36) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 37) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 38) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 39) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 40) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 41) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 42) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 43) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 44) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 45) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 46) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 47) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 48) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 49) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 50) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 51) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 52) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 53) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 54) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 55) ? 1'b1
			: ({ S , CN[i], A[i] , B[i] } == 56) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 57) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 58) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 59) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 60) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 61) ? 1'b0
			: ({ S , CN[i], A[i] , B[i] } == 62) ? 1'b0
			: 1'b1;
`endif
	   end
    endgenerate

    assign C_OUT = CN[ALU_WIDTH];

`endif

endmodule

/*
 *   S   Cn   A   B    |   F0  Fn   OVR  Cn+1  G   P
 * ------------------------------------------------------
 *   0    0   x   x    |   0   0     1    1    0   0    |
 *   0    0   x   x    |   0   0     1    1    0   0    |
 *   0    0   x   x    |   0   0     1    1    0   0    | 
 *   0    0   x   x    |   0   0     1    1    0   0    | CLEAR
 *   0    1   x   x    |   0   0     1    1    0   0    | 
 *   0    1   x   x    |   0   0     1    1    0   0    | 
 *   0    1   x   x    |   0   0     1    1    0   0    | 
 *   0    1   x   x    |   0   0     1    1    0   0    | 
 * ------------------------------------------------------
 *   1    0   0   0    |   1   1     0    0    1   0    |
 *   1    0   0   1    |   0   1     0    1    0   0    |
 *   1    0   1   0    |   0   0     0    0    1   1    |
 *   1    0   1   1    |   1   1     0    0    1   0    | B - A - ~Cn
 *   1    1   0   0    |   0   0     0    1    1   0    |
 *   1    1   0   1    |   1   1     0    1    0   0    |
 *   1    1   1   0    |   1   0     0    0    1   1    |
 *   1    1   1   1    |   0   0     0    1    1   0    |
 * ------------------------------------------------------
 *   2    0   0   0    |   1   1     0    0    1   0    |
 *   2    0   0   1    |   0   0     0    0    1   1    |
 *   2    0   1   0    |   0   1     0    1    0   0    |
 *   2    0   1   1    |   1   1     0    0    1   0    | A - B - ~Cn
 *   2    1   0   0    |   0   0     0    1    1   0    |
 *   2    1   0   1    |   1   0     0    0    1   1    |
 *   2    1   1   0    |   1   1     0    1    0   0    |
 *   2    1   1   1    |   0   0     0    1    1   0    |
 * ------------------------------------------------------
 *   3    0   0   0    |   0   0     0    0    1   1    |
 *   3    0   0   1    |   1   1     0    0    1   0    |
 *   3    0   1   0    |   1   1     0    0    1   0    |
 *   3    0   1   1    |   0   1     0    1    0   0    | A + B + Cn
 *   3    1   0   0    |   1   0     0    0    1   1    |
 *   3    1   0   1    |   0   0     0    1    1   0    |
 *   3    1   1   0    |   0   0     0    1    1   0    |
 *   3    1   1   1    |   1   1     0    1    0   0    |
 * ------------------------------------------------------
 *   4    X   0   0    |   0   <     0    0    1   1    |
 *   4    X   0   1    |   1   <     0    0    1   1    |
 *   4    0   1   0    |   1   <     0    0    1   0    |
 *   4    X   1   1    |   0   <     1    1    0   0    | A ^ B 
 *   4    X   0   0    |   0   <     0    0    1   1    |
 *   4    X   0   1    |   1   <     0    0    1   1    |
 *   4    1   1   0    |   1   <     1    1    1   0    |
 *   4    X   1   1    |   0   <     1    1    0   0    | 
 * ------------------------------------------------------
 *   5    X   0   0    |   0   <     0    0    1   1    |
 *   5    X   0   1    |   1   <     0    0    1   1    |
 *   5    X   1   0    |   1   <     0    0    1   1    |
 *   5    0   1   1    |   1   <     0    0    1   0    | A | B 
 *   5    X   0   0    |   0   <     0    0    1   1    |
 *   5    X   0   1    |   1   <     0    0    1   1    |
 *   5    X   1   0    |   1   <     0    0    1   1    |
 *   5    1   1   1    |   1   <     1    1    1   0    |
 * ------------------------------------------------------
 *   6    X   0   0    |   0   <     1    1    0   0    |
 *   6    X   0   1    |   0   <     0    0    1   1    |
 *   6    X   1   0    |   0   <     1    1    0   0    |
 *   6    0   1   1    |   1   <     0    0    1   0    | A & B 
 *   6    X   0   0    |   0   <     1    1    0   0    |
 *   6    X   0   1    |   0   <     0    0    1   1    |
 *   6    X   1   0    |   0   <     1    1    0   0    |
 *   6    1   1   1    |   1   <     1    1    1   0    |
 * ------------------------------------------------------
 *   7    X   0   0    |   1   <     0    0    1   1    |
 *   7    X   0   1    |   1   <     0    0    1   1    |
 *   7    X   1   0    |   1   <     0    0    1   1    |
 *   7    0   1   1    |   1   <     0    0    1   0    | Rreset
 *   7    X   0   0    |   1   <     0    0    1   1    |
 *   7    X   0   1    |   1   <     0    0    1   1    |
 *   7    X   1   0    |   1   <     0    0    1   1    |
 *   7    1   1   1    |   1   <     1    1    1   0    |
 * ------------------------------------------------------

bit pattern

F0     00000000 10010110 10010110 01101001 01100110 01110111 00010001 11111111
Fn     00000000 11010100 10110010 01110001 01100110 01110111 00010001 11111111
OVR    11111111 00000000 00000000 00000000 00010011 00000001 10101011 00000001
Cn+1   11111111 01001101 00101011 00010111 00010011 00000001 10101011 00000001
G      00000000 10111011 11011101 11101110 11101110 11111111 01010101 11111111
P      00000000 00100010 01000100 10001000 11001100 11101110 01000100 11101110

*/
