/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

`define SHRINKED_LUT

module alu_74382 # (
	parameter ALU_WIDTH = 4
) (
		  input   C_IN       // Carry Input
		, input  [ALU_WIDTH -1:0] A    // DATA IN
		, input  [ALU_WIDTH -1:0] B    // DATA IN
		, output [ALU_WIDTH -1:0] F    // DATA OUT
		, output  C_OUT      // carry-look-ahead
		, output  OVR        // overflow
		, input  [2:0] S   // Function Select

//		, input   CP       // Clock
		);


//`define ZZZ
//`define YYY
`ifdef ZZZ
    wire [ALU_WIDTH -2:0] minus1 = -1;
    wire [ALU_WIDTH:0] Rd_out =
		  (S == 1) ? B - A - C_IN
		: (S == 2) ? A - B - C_IN
		: (S == 3) ? A + B + C_IN
		: (S == 4) ? { 1'b0,  (A ^ B) } 
		: (S == 5) ? { 1'b0,  (A | B) }
		: (S == 6) ? { 1'b0,  (A & B) }
		: (S == 7) ? { 1'b0, minus1 , C_IN }
		: 0;

    assign F     = Rd_out[ALU_WIDTH -1:0];
    assign C_OUT = Rd_out[ALU_WIDTH];

`elsif YYY
    wire [ALU_WIDTH -2:0] minus1 = -1;
    wire [ALU_WIDTH -1:0] Rd_in = (S == 1) ? B : A;
    wire [ALU_WIDTH -1:0] Rr_in = (S == 1) ? A : B;

    wire [ALU_WIDTH:0] Rd_out = 
		 ((S == 1) | (S == 2)) ? Rd_in  - Rd_in - C_IN
		: (S == 3) ? Rd_in + Rr_in + C_IN
		: (S == 4) ? { 1'b0,  (Rd_in ^ Rr_in) } 
		: (S == 5) ? { 1'b0,  (Rd_in | Rr_in) }
		: (S == 6) ? { 1'b0,  (Rd_in & Rr_in) }
		: (S == 7) ? { 1'b0, minus1 , C_IN }
		: 0;

   assign OVR = (S == 3) ? (Rd_out[ALU_WIDTH -1] 
				& ~Rd_in[ALU_WIDTH -1] & ~Rr_in[ALU_WIDTH -1]
                	    | ~Rd_out[ALU_WIDTH -1] 
				&  Rd_in[ALU_WIDTH -1] &  Rr_in[ALU_WIDTH -1] )
	      : ( (S == 1) | (S == 2) )
			? (Rd_out[ALU_WIDTH -1]
				 & ~Rd_in[ALU_WIDTH -1] &  Rr_in[ALU_WIDTH -1]
               			| ~Rd_out[ALU_WIDTH -1]
				 &  Rd_in[ALU_WIDTH -1] & ~Rr_in[ALU_WIDTH -1] )
	      : 0 ;

    assign F     = Rd_out[ALU_WIDTH -1:0];
    assign C_OUT = Rd_out[ALU_WIDTH];
`else // using LUT
     // input { S , C_IN, A , B }
     // output { F , C_OUT , OVR }
    reg lut_F0 [63: 0];
`ifdef SHRINKED_LUT
    reg lut_Cn [31: 0];
    reg lut_OV [31: 0];
`else
    reg lut_Cn [63: 0];
    reg lut_OV [63: 0];
`endif

//    wire [ALU_WIDTH:0] F;
//    wire C_OUT;
//    wire OVR;
    wire [ALU_WIDTH:0] CN;

    assign CN[0]  = C_IN;
    generate
       genvar i;
       for (i = 0; i < ALU_WIDTH; i=i+1)
           begin : gen_lut
		assign F[i]     = lut_F0[{ S , CN[i], A[i] , B[i] }];
`ifdef SHRINKED_LUT
		assign CN[i+1]  = S[2] ? 0 
				  : lut_Cn[{ S[1:0] , CN[i], A[i] , B[i] }];
`else
		assign CN[i+1]  = lut_Cn[{ S , CN[i], A[i] , B[i] }];
`endif
	   end
    endgenerate

    assign C_OUT = CN[ALU_WIDTH];

`ifdef SHRINKED_LUT
    assign OVR = S[2] ? 0 : lut_OV[{ S[1:0] , CN[ALU_WIDTH -1]
				, A[ALU_WIDTH -1] , B[ALU_WIDTH -1] }];
`else
    assign OVR = lut_OV[{ S , CN[ALU_WIDTH -1]
				, A[ALU_WIDTH -1] , B[ALU_WIDTH -1] }];
`endif
    initial
        begin : build_lut
		lut_F0[ 0] <= 1'b0;	lut_F0[ 1] <= 1'b0;
		lut_F0[ 2] <= 1'b0;	lut_F0[ 3] <= 1'b0;
		lut_F0[ 4] <= 1'b0;	lut_F0[ 5] <= 1'b0;
		lut_F0[ 6] <= 1'b0;	lut_F0[ 7] <= 1'b0;
		lut_F0[ 8] <= 1'b1;	lut_F0[ 9] <= 1'b0;
		lut_F0[10] <= 1'b0;	lut_F0[11] <= 1'b1;
		lut_F0[12] <= 1'b0;	lut_F0[13] <= 1'b1;
		lut_F0[14] <= 1'b1;	lut_F0[15] <= 1'b0;
		lut_F0[16] <= 1'b1;	lut_F0[17] <= 1'b0;
		lut_F0[18] <= 1'b0;	lut_F0[19] <= 1'b1;
		lut_F0[20] <= 1'b0;	lut_F0[21] <= 1'b1;
		lut_F0[22] <= 1'b1;	lut_F0[23] <= 1'b0;
		lut_F0[24] <= 1'b0;	lut_F0[25] <= 1'b1;
		lut_F0[26] <= 1'b1;	lut_F0[27] <= 1'b0;
		lut_F0[28] <= 1'b1;	lut_F0[29] <= 1'b0;
		lut_F0[30] <= 1'b0;	lut_F0[31] <= 1'b1;
		lut_F0[32] <= 1'b0;	lut_F0[33] <= 1'b1;
		lut_F0[34] <= 1'b1;	lut_F0[35] <= 1'b0;
		lut_F0[36] <= 1'b0;	lut_F0[37] <= 1'b1;
		lut_F0[38] <= 1'b1;	lut_F0[39] <= 1'b0;
		lut_F0[40] <= 1'b0;	lut_F0[41] <= 1'b1;
		lut_F0[42] <= 1'b1;	lut_F0[43] <= 1'b1;
		lut_F0[44] <= 1'b0;	lut_F0[45] <= 1'b1;
		lut_F0[46] <= 1'b1;	lut_F0[47] <= 1'b1;
		lut_F0[48] <= 1'b0;	lut_F0[49] <= 1'b0;
		lut_F0[50] <= 1'b0;	lut_F0[51] <= 1'b1;
		lut_F0[52] <= 1'b0;	lut_F0[53] <= 1'b0;
		lut_F0[54] <= 1'b0;	lut_F0[55] <= 1'b1;
		lut_F0[56] <= 1'b1;	lut_F0[57] <= 1'b1;
		lut_F0[58] <= 1'b1;	lut_F0[59] <= 1'b1;
		lut_F0[60] <= 1'b1;	lut_F0[61] <= 1'b1;
		lut_F0[62] <= 1'b1;	lut_F0[63] <= 1'b1;
`ifdef NOT_USED
		lut_Fn[ 0] <= 1'b0;	lut_Fn[ 1] <= 1'b0;
		lut_Fn[ 2] <= 1'b0;	lut_Fn[ 3] <= 1'b0;
		lut_Fn[ 4] <= 1'b0;	lut_Fn[ 5] <= 1'b0;
		lut_Fn[ 6] <= 1'b0;	lut_Fn[ 7] <= 1'b0;
		lut_Fn[ 8] <= 1'b1;	lut_Fn[ 9] <= 1'b1;
		lut_Fn[10] <= 1'b0;	lut_Fn[11] <= 1'b1;
		lut_Fn[12] <= 1'b0;	lut_Fn[13] <= 1'b1;
		lut_Fn[14] <= 1'b0;	lut_Fn[15] <= 1'b0;
		lut_Fn[16] <= 1'b1;	lut_Fn[17] <= 1'b0;
		lut_Fn[18] <= 1'b1;	lut_Fn[19] <= 1'b1;
		lut_Fn[20] <= 1'b0;	lut_Fn[21] <= 1'b0;
		lut_Fn[22] <= 1'b1;	lut_Fn[23] <= 1'b0;
		lut_Fn[24] <= 1'b0;	lut_Fn[25] <= 1'b1;
		lut_Fn[26] <= 1'b1;	lut_Fn[27] <= 1'b1;
		lut_Fn[28] <= 1'b0;	lut_Fn[29] <= 1'b0;
		lut_Fn[30] <= 1'b0;	lut_Fn[31] <= 1'b1;
		lut_Fn[32] <= 1'b0;	lut_Fn[33] <= 1'b1;
		lut_Fn[34] <= 1'b1;	lut_Fn[35] <= 1'b0;
		lut_Fn[36] <= 1'b0;	lut_Fn[37] <= 1'b1;
		lut_Fn[38] <= 1'b1;	lut_Fn[39] <= 1'b0;
		lut_Fn[40] <= 1'b0;	lut_Fn[41] <= 1'b1;
		lut_Fn[42] <= 1'b1;	lut_Fn[43] <= 1'b1;
		lut_Fn[44] <= 1'b0;	lut_Fn[45] <= 1'b1;
		lut_Fn[46] <= 1'b1;	lut_Fn[47] <= 1'b1;
		lut_Fn[48] <= 1'b0;	lut_Fn[49] <= 1'b0;
		lut_Fn[50] <= 1'b0;	lut_Fn[51] <= 1'b1;
		lut_Fn[52] <= 1'b0;	lut_Fn[53] <= 1'b0;
		lut_Fn[54] <= 1'b0;	lut_Fn[55] <= 1'b1;
		lut_Fn[56] <= 1'b1;	lut_Fn[57] <= 1'b1;
		lut_Fn[58] <= 1'b1;	lut_Fn[59] <= 1'b1;
		lut_Fn[60] <= 1'b1;	lut_Fn[61] <= 1'b1;
		lut_Fn[62] <= 1'b1;	lut_Fn[63] <= 1'b1;
`endif
		lut_OV[ 0] <= 1'b1;	lut_OV[ 1] <= 1'b1;
		lut_OV[ 2] <= 1'b1;	lut_OV[ 3] <= 1'b1;
		lut_OV[ 4] <= 1'b1;	lut_OV[ 5] <= 1'b1;
		lut_OV[ 6] <= 1'b1;	lut_OV[ 7] <= 1'b1;
		lut_OV[ 8] <= 1'b0;	lut_OV[ 9] <= 1'b0;
		lut_OV[10] <= 1'b0;	lut_OV[11] <= 1'b0;
		lut_OV[12] <= 1'b0;	lut_OV[13] <= 1'b0;
		lut_OV[14] <= 1'b0;	lut_OV[15] <= 1'b0;
		lut_OV[16] <= 1'b0;	lut_OV[17] <= 1'b0;
		lut_OV[18] <= 1'b0;	lut_OV[19] <= 1'b0;
		lut_OV[20] <= 1'b0;	lut_OV[21] <= 1'b0;
		lut_OV[22] <= 1'b0;	lut_OV[23] <= 1'b0;
		lut_OV[24] <= 1'b0;	lut_OV[25] <= 1'b0;
		lut_OV[26] <= 1'b0;	lut_OV[27] <= 1'b0;
		lut_OV[28] <= 1'b0;	lut_OV[29] <= 1'b0;
		lut_OV[30] <= 1'b0;	lut_OV[31] <= 1'b0;
`ifdef NOT_USED
		lut_OV[32] <= 1'b0;	lut_OV[33] <= 1'b0;
		lut_OV[34] <= 1'b0;	lut_OV[35] <= 1'b1;
		lut_OV[36] <= 1'b0;	lut_OV[37] <= 1'b0;
		lut_OV[38] <= 1'b1;	lut_OV[39] <= 1'b1;
		lut_OV[40] <= 1'b0;	lut_OV[41] <= 1'b0;
		lut_OV[42] <= 1'b0;	lut_OV[43] <= 1'b0;
		lut_OV[44] <= 1'b0;	lut_OV[45] <= 1'b0;
		lut_OV[46] <= 1'b0;	lut_OV[47] <= 1'b1;
		lut_OV[48] <= 1'b1;	lut_OV[49] <= 1'b0;
		lut_OV[50] <= 1'b1;	lut_OV[51] <= 1'b0;
		lut_OV[52] <= 1'b1;	lut_OV[53] <= 1'b0;
		lut_OV[54] <= 1'b1;	lut_OV[55] <= 1'b1;
		lut_OV[56] <= 1'b0;	lut_OV[57] <= 1'b0;
		lut_OV[58] <= 1'b0;	lut_OV[59] <= 1'b0;
		lut_OV[60] <= 1'b0;	lut_OV[61] <= 1'b0;
		lut_OV[62] <= 1'b0;	lut_OV[63] <= 1'b1;
`endif
	
		lut_Cn[ 0] <= 1'b1;	lut_Cn[ 1] <= 1'b1;
		lut_Cn[ 2] <= 1'b1;	lut_Cn[ 3] <= 1'b1;
		lut_Cn[ 4] <= 1'b1;	lut_Cn[ 5] <= 1'b1;
		lut_Cn[ 6] <= 1'b1;	lut_Cn[ 7] <= 1'b1;
		lut_Cn[ 8] <= 1'b0;	lut_Cn[ 9] <= 1'b1;
		lut_Cn[10] <= 1'b0;	lut_Cn[11] <= 1'b0;
		lut_Cn[12] <= 1'b1;	lut_Cn[13] <= 1'b1;
		lut_Cn[14] <= 1'b0;	lut_Cn[15] <= 1'b1;
		lut_Cn[16] <= 1'b0;	lut_Cn[17] <= 1'b0;
		lut_Cn[18] <= 1'b1;	lut_Cn[19] <= 1'b0;
		lut_Cn[20] <= 1'b1;	lut_Cn[21] <= 1'b0;
		lut_Cn[22] <= 1'b1;	lut_Cn[23] <= 1'b1;
		lut_Cn[24] <= 1'b0;	lut_Cn[25] <= 1'b0;
		lut_Cn[26] <= 1'b0;	lut_Cn[27] <= 1'b1;
		lut_Cn[28] <= 1'b0;	lut_Cn[29] <= 1'b1;
		lut_Cn[30] <= 1'b1;	lut_Cn[31] <= 1'b1;
`ifdef NOT_USED
		lut_Cn[32] <= 1'b0;	lut_Cn[33] <= 1'b0;
		lut_Cn[34] <= 1'b0;	lut_Cn[35] <= 1'b1;
		lut_Cn[36] <= 1'b0;	lut_Cn[37] <= 1'b0;
		lut_Cn[38] <= 1'b1;	lut_Cn[39] <= 1'b1;
		lut_Cn[40] <= 1'b0;	lut_Cn[41] <= 1'b0;
		lut_Cn[42] <= 1'b0;	lut_Cn[43] <= 1'b0;
		lut_Cn[44] <= 1'b0;	lut_Cn[45] <= 1'b0;
		lut_Cn[46] <= 1'b0;	lut_Cn[47] <= 1'b1;
		lut_Cn[48] <= 1'b1;	lut_Cn[49] <= 1'b0;
		lut_Cn[50] <= 1'b1;	lut_Cn[51] <= 1'b0;
		lut_Cn[52] <= 1'b1;	lut_Cn[53] <= 1'b0;
		lut_Cn[54] <= 1'b1;	lut_Cn[55] <= 1'b1;
		lut_Cn[56] <= 1'b0;	lut_Cn[57] <= 1'b0;
		lut_Cn[58] <= 1'b0;	lut_Cn[59] <= 1'b0;
		lut_Cn[60] <= 1'b0;	lut_Cn[61] <= 1'b0;
		lut_Cn[62] <= 1'b0;	lut_Cn[63] <= 1'b1;
`endif
`ifdef SUPPORT_PG	
		lut_G[ 0] <= 1'b0;	lut_G[ 1] <= 1'b0;
		lut_G[ 2] <= 1'b0;	lut_G[ 3] <= 1'b0;
		lut_G[ 4] <= 1'b0;	lut_G[ 5] <= 1'b0;
		lut_G[ 6] <= 1'b0;	lut_G[ 7] <= 1'b0;
		lut_G[ 8] <= 1'b1;	lut_G[ 9] <= 1'b0;
		lut_G[10] <= 1'b1;	lut_G[11] <= 1'b1;
		lut_G[12] <= 1'b1;	lut_G[13] <= 1'b0;
		lut_G[14] <= 1'b1;	lut_G[15] <= 1'b1;
		lut_G[16] <= 1'b1;	lut_G[17] <= 1'b1;
		lut_G[18] <= 1'b0;	lut_G[19] <= 1'b1;
		lut_G[20] <= 1'b1;	lut_G[21] <= 1'b1;
		lut_G[22] <= 1'b0;	lut_G[23] <= 1'b1;
		lut_G[24] <= 1'b1;	lut_G[25] <= 1'b1;
		lut_G[26] <= 1'b1;	lut_G[27] <= 1'b0;
		lut_G[28] <= 1'b1;	lut_G[29] <= 1'b1;
		lut_G[30] <= 1'b1;	lut_G[31] <= 1'b0;
		lut_G[32] <= 1'b1;	lut_G[33] <= 1'b1;
		lut_G[34] <= 1'b1;	lut_G[35] <= 1'b0;
		lut_G[36] <= 1'b1;	lut_G[37] <= 1'b1;
		lut_G[38] <= 1'b1;	lut_G[39] <= 1'b0;
		lut_G[40] <= 1'b1;	lut_G[41] <= 1'b1;
		lut_G[42] <= 1'b1;	lut_G[43] <= 1'b1;
		lut_G[44] <= 1'b1;	lut_G[45] <= 1'b1;
		lut_G[46] <= 1'b1;	lut_G[47] <= 1'b1;
		lut_G[48] <= 1'b0;	lut_G[49] <= 1'b1;
		lut_G[50] <= 1'b0;	lut_G[51] <= 1'b1;
		lut_G[52] <= 1'b0;	lut_G[53] <= 1'b1;
		lut_G[54] <= 1'b0;	lut_G[55] <= 1'b1;
		lut_G[56] <= 1'b1;	lut_G[57] <= 1'b1;
		lut_G[58] <= 1'b1;	lut_G[59] <= 1'b1;
		lut_G[60] <= 1'b1;	lut_G[61] <= 1'b1;
		lut_G[62] <= 1'b1;	lut_G[63] <= 1'b1;
	
		lut_P[ 0] <= 1'b0;	lut_P[ 1] <= 1'b0;
		lut_P[ 2] <= 1'b0;	lut_P[ 3] <= 1'b0;
		lut_P[ 4] <= 1'b0;	lut_P[ 5] <= 1'b0;
		lut_P[ 6] <= 1'b0;	lut_P[ 7] <= 1'b0;
		lut_P[ 8] <= 1'b0;	lut_P[ 9] <= 1'b0;
		lut_P[10] <= 1'b1;	lut_P[11] <= 1'b0;
		lut_P[12] <= 1'b0;	lut_P[13] <= 1'b0;
		lut_P[14] <= 1'b1;	lut_P[15] <= 1'b0;
		lut_P[16] <= 1'b0;	lut_P[17] <= 1'b1;
		lut_P[18] <= 1'b0;	lut_P[19] <= 1'b0;
		lut_P[20] <= 1'b0;	lut_P[21] <= 1'b1;
		lut_P[22] <= 1'b0;	lut_P[23] <= 1'b0;
		lut_P[24] <= 1'b1;	lut_P[25] <= 1'b0;
		lut_P[26] <= 1'b0;	lut_P[27] <= 1'b0;
		lut_P[28] <= 1'b1;	lut_P[29] <= 1'b0;
		lut_P[30] <= 1'b0;	lut_P[31] <= 1'b0;
		lut_P[32] <= 1'b1;	lut_P[33] <= 1'b1;
		lut_P[34] <= 1'b0;	lut_P[35] <= 1'b0;
		lut_P[36] <= 1'b1;	lut_P[37] <= 1'b1;
		lut_P[38] <= 1'b0;	lut_P[39] <= 1'b0;
		lut_P[40] <= 1'b1;	lut_P[41] <= 1'b1;
		lut_P[42] <= 1'b1;	lut_P[43] <= 1'b0;
		lut_P[44] <= 1'b1;	lut_P[45] <= 1'b1;
		lut_P[46] <= 1'b1;	lut_P[47] <= 1'b0;
		lut_P[48] <= 1'b0;	lut_P[49] <= 1'b1;
		lut_P[50] <= 1'b0;	lut_P[51] <= 1'b0;
		lut_P[52] <= 1'b0;	lut_P[53] <= 1'b1;
		lut_P[54] <= 1'b0;	lut_P[55] <= 1'b0;
		lut_P[56] <= 1'b1;	lut_P[57] <= 1'b1;
		lut_P[58] <= 1'b1;	lut_P[59] <= 1'b0;
		lut_P[60] <= 1'b1;	lut_P[61] <= 1'b1;
		lut_P[62] <= 1'b1;	lut_P[63] <= 1'b0;
`endif	
	end

`endif

endmodule

/*
 *   S   Cn   A   B    |   F0  Fn   OVR  Cn+1  G   P
 * ------------------------------------------------------
 *   0    0   x   x    |   0   0     1    1    0   0    |
 *   0    0   x   x    |   0   0     1    1    0   0    |
 *   0    0   x   x    |   0   0     1    1    0   0    | 
 *   0    0   x   x    |   0   0     1    1    0   0    | CLEAR
 *   0    1   x   x    |   0   0     1    1    0   0    | 
 *   0    1   x   x    |   0   0     1    1    0   0    | 
 *   0    1   x   x    |   0   0     1    1    0   0    | 
 *   0    1   x   x    |   0   0     1    1    0   0    | 
 * ------------------------------------------------------
 *   1    0   0   0    |   1   1     0    0    1   0    |
 *   1    0   0   1    |   0   1     0    1    0   0    |
 *   1    0   1   0    |   0   0     0    0    1   1    |
 *   1    0   1   1    |   1   1     0    0    1   0    | B - A - ~Cn
 *   1    1   0   0    |   0   0     0    1    1   0    |
 *   1    1   0   1    |   1   1     0    1    0   0    |
 *   1    1   1   0    |   1   0     0    0    1   1    |
 *   1    1   1   1    |   0   0     0    1    1   0    |
 * ------------------------------------------------------
 *   2    0   0   0    |   1   1     0    0    1   0    |
 *   2    0   0   1    |   0   0     0    0    1   1    |
 *   2    0   1   0    |   0   1     0    1    0   0    |
 *   2    0   1   1    |   1   1     0    0    1   0    | A - B - ~Cn
 *   2    1   0   0    |   0   0     0    1    1   0    |
 *   2    1   0   1    |   1   0     0    0    1   1    |
 *   2    1   1   0    |   1   1     0    1    0   0    |
 *   2    1   1   1    |   0   0     0    1    1   0    |
 * ------------------------------------------------------
 *   3    0   0   0    |   0   0     0    0    1   1    |
 *   3    0   0   1    |   1   1     0    0    1   0    |
 *   3    0   1   0    |   1   1     0    0    1   0    |
 *   3    0   1   1    |   0   1     0    1    0   0    | A + B + Cn
 *   3    1   0   0    |   1   0     0    0    1   1    |
 *   3    1   0   1    |   0   0     0    1    1   0    |
 *   3    1   1   0    |   0   0     0    1    1   0    |
 *   3    1   1   1    |   1   1     0    1    0   0    |
 * ------------------------------------------------------
 *   4    X   0   0    |   0   <     0    0    1   1    |
 *   4    X   0   1    |   1   <     0    0    1   1    |
 *   4    0   1   0    |   1   <     0    0    1   0    |
 *   4    X   1   1    |   0   <     1    1    0   0    | A ^ B 
 *   4    X   0   0    |   0   <     0    0    1   1    |
 *   4    X   0   1    |   1   <     0    0    1   1    |
 *   4    1   1   0    |   1   <     1    1    1   0    |
 *   4    X   1   1    |   0   <     1    1    0   0    | 
 * ------------------------------------------------------
 *   5    X   0   0    |   0   <     0    0    1   1    |
 *   5    X   0   1    |   1   <     0    0    1   1    |
 *   5    X   1   0    |   1   <     0    0    1   1    |
 *   5    0   1   1    |   1   <     0    0    1   0    | A | B 
 *   5    X   0   0    |   0   <     0    0    1   1    |
 *   5    X   0   1    |   1   <     0    0    1   1    |
 *   5    X   1   0    |   1   <     0    0    1   1    |
 *   5    1   1   1    |   1   <     1    1    1   0    |
 * ------------------------------------------------------
 *   6    X   0   0    |   0   <     1    1    0   0    |
 *   6    X   0   1    |   0   <     0    0    1   1    |
 *   6    X   1   0    |   0   <     1    1    0   0    |
 *   6    0   1   1    |   1   <     0    0    1   0    | A & B 
 *   6    X   0   0    |   0   <     1    1    0   0    |
 *   6    X   0   1    |   0   <     0    0    1   1    |
 *   6    X   1   0    |   0   <     1    1    0   0    |
 *   6    1   1   1    |   1   <     1    1    1   0    |
 * ------------------------------------------------------
 *   7    X   0   0    |   1   <     0    0    1   1    |
 *   7    X   0   1    |   1   <     0    0    1   1    |
 *   7    X   1   0    |   1   <     0    0    1   1    |
 *   7    0   1   1    |   1   <     0    0    1   0    | Rreset
 *   7    X   0   0    |   1   <     0    0    1   1    |
 *   7    X   0   1    |   1   <     0    0    1   1    |
 *   7    X   1   0    |   1   <     0    0    1   1    |
 *   7    1   1   1    |   1   <     1    1    1   0    |
 * ------------------------------------------------------

bit pattern

F0     00000000 10010110 10010110 01101001 01100110 01110111 00010001 11111111
Fn     00000000 11010100 10110010 01110001 01100110 01110111 00010001 11111111
OVR    11111111 00000000 00000000 00000000 00010011 00000001 10101011 00000001
Cn+1   11111111 01001101 00101011 00010111 00010011 00000001 10101011 00000001
G      00000000 10111011 11011101 11101110 11101110 11111111 01010101 11111111
P      00000000 00100010 01000100 10001000 11001100 11101110 01000100 11101110

*/
