/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

`include "rtavr_defs.v"
// global defines : OUTPUT_MUX_TRI

//`define OUTPUT_MUX_TRI

// Module Name: rtavr_ior_timer0
//   version 002
// 
// Target Device:   xc3s200a-4ft256
// Product Version: ISE 12.4
// Design Goal:     Balanced
//
//                          DO:  NORMAL    TRISTATE(FAKE)   
// Number of Slice Flip Flops:     49         49   
// Number of 4 input LUTs:         69         73
// Number of occupied Slices:      56         57
// Total Number of 4 input LUTs:   78         82 
// Number of bonded IOBs :         28
//   IOB Flip Flops :              -
// Number of BUFGMUXs :             1
// Number of RAMB16BWEs :          -

module rtavr_ior_timer0(
		input CLK,
		input RESET,

		input [7:0] DI
`ifdef IOR_LARGE_MAPPER
		, input WE_ocr0b
		, input WE_ocr0a
		, input WE_tcnt0
		, input WE_tccr0b
		, input WE_tccr0a
		, output [7:0] DO_ocr0b
		, output [7:0] DO_ocr0a
		, output [7:0] DO_tcnt0
		, output [7:0] DO_tccr0b
		, output [7:0] DO_tccr0a
`else
		, input WR,RD
		, input [5:0] ADDR
		, output [7:0] DO
`endif
		// external I/O
		, output TIM0_OVF
		, output TIM0_COMPA	
		, output TIM0_COMPB
`ifdef IOR_HAVE_TIM0_OC0A
		, output OC0A_EN
		, output OC0A_OUT
`endif
`ifdef IOR_HAVE_TIM0_OC0B
		, output OC0B_EN
		, output OC0B_OUT
`endif
`ifdef TIM0_EXPORT_PS
		, output [9:0] PS
`endif
		);
  parameter BASE_ADDR = 6'h15; // OCR0B 0x15
  reg  [9:0] r_prescaler;
  reg  [7:0] r_tcnt0;
`ifdef FIXED_OCR0B
  wire [7:0] r_ocr0b = `FIXED_OCR0B;
`else
  reg  [7:0] r_ocr0b;
`endif
`ifdef FIXED_OCR0A
  wire [7:0] r_ocr0a = `FIXED_OCR0A;
`else
  reg  [7:0] r_ocr0a;
`endif
  reg  r_tim0_compa;
  reg  r_tim0_compb;
  reg  r_tim0_ovf;
 
  assign TIM0_OVF = r_tim0_ovf;
  assign TIM0_COMPA = r_tim0_compa;
  assign TIM0_COMPB = r_tim0_compb;

`ifdef TIMER0_EXPORT_PS
  assign PS         = r_prescaler;
`endif

`ifdef IOR_LARGE_MAPPER
`else
      wire OE_ocr0b = RD & (ADDR == BASE_ADDR);
      wire WE_ocr0b = WR & (ADDR == BASE_ADDR);
      wire OE_ocr0a = RD & (ADDR == (BASE_ADDR + 1));
      wire WE_ocr0a = WR & (ADDR == (BASE_ADDR + 1));
      wire OE_tcnt0 = RD & (ADDR == (BASE_ADDR + 2));
      wire WE_tcnt0 = WR & (ADDR == (BASE_ADDR + 2));
      wire OE_tccr0b = RD & (ADDR == (BASE_ADDR + 3));
      wire WE_tccr0b = WR & (ADDR == (BASE_ADDR + 3));
      wire OE_tccr0a = RD & (ADDR == (BASE_ADDR + 4));
      wire WE_tccr0a = WR & (ADDR == (BASE_ADDR + 4));
`endif // IOR_LARGE_MAPPER

`ifdef FIXED_CS0
  reg r_cs0;
  wire [2:0] CS0 = r_cs0 ? 3'b0 : `FIXED_CS0;
`else
  reg [2:0] CS0;
`endif
`ifdef FIXED_WGM0
  wire [2:0] WGM0 = `FIXED_WGM0;
`else
  reg [2:0] WGM0;
`endif
`ifdef FIXED_COM0A
  wire [1:0] COM0A = `FIXED_COM0A;
`else
  reg [1:0] COM0A;
`endif
`ifdef FIXED_COM0B
  wire [1:0] COM0B = `FIXED_COM0B;
`else
  reg [1:0] COM0B;
`endif

`ifdef IOR_HAVE_TIM0_OC0A
  reg r_oc0a;
  assign OC0A_EN = (COM0A != 0);
  assign OC0A_OUT = r_oc0a;
`endif
`ifdef IOR_HAVE_TIM0_OC0B
  reg r_oc0b;
  assign OC0B_EN = (COM0B != 0);
  assign OC0B_OUT = r_oc0b;
`endif

  reg tsm;
  reg psr;
`ifdef TIM0_HAVE_FOC
  reg foc0b;
  reg foc0a;
`endif

  wire  [7:0] i_tccr0a = {COM0A, COM0B, WGM0[2], CS0};
  wire  [7:0] i_tccr0b = {2'b0, tsm, 3'b0 , WGM0[1:0]};

  wire v_psr = (WE_tccr0b & DI[4]) | (~WE_tccr0b & tsm & psr);

  reg t0_prev;
  wire t0_nops = (CS0 == 1);
  wire t0_clk =  (CS0 == 2) ? r_prescaler[2] // 1/8
               : (CS0 == 3) ? r_prescaler[5] // 1/64
               : (CS0 == 4) ? r_prescaler[7] // 1/256
               : (CS0 == 5) ? r_prescaler[9] // 1/1024
               : 0;
  wire t0_top_ocr0a = ((WGM0 == 2) | (WGM0 == 5) | (WGM0 == 7));
`ifdef NOT_YET
  wire ocr0_chg_top = ((WGM0 == 1) | (WGM0 == 5));
  wire ocr0_chg_btm = ((WGM0 == 3) | (WGM0 == 7));
  wire ocr0_chg_imm = (WGM0[0] == 0);
`endif
`ifdef IOR_LARGE_MAPPER
	assign DO_ocr0b[7:0] =  r_ocr0b[7:0];
	assign DO_ocr0a[7:0] =  r_ocr0a[7:0];
	assign DO_tcnt0[7:0] =  r_tcnt0[7:0];
	assign DO_tccr0b[7:0] = i_tccr0b[7:0];
	assign DO_tccr0a[7:0] = i_tccr0a[7:0];
`else // IOR_LARGE_MAPPER
      wire [7:0] i_DO = OE_ocr0b ? r_ocr0b : OE_ocr0a ? r_ocr0a 
		  : OE_tcnt0 ? r_tcnt0 : OE_tccr0b ? i_tccr0b 
  `ifdef OUTPUT_MUX_OR
		  : OE_tccr0a ? i_tccr0a : 0 ;
  `else
		  : i_tccr0a ;
  `endif
  `ifdef OUTPUT_MUX_TRI
      bufif1(DO,i_DO, (OE_ocr0b | OE_ocr0a | OE_tcnt0 | OE_tccr0b | OE_tccr0a) );
  `else
      assign DO = i_DO;
  `endif
`endif // IOR_LARGE_MAPPER

  always @(negedge CLK)
  begin
    if (~RESET | v_psr) r_prescaler <= 1;
    else              r_prescaler <= r_prescaler + 10'h001;
  end

  always @(posedge CLK)
  begin
    if (RESET == 0)
      begin
`ifdef FIXED_OCR0B
`else
	r_ocr0b <= 0; 	 
`endif
`ifdef FIXED_OCR0A
`else
	r_ocr0a <= 0;
`endif
`ifdef FIXED_CS0
	r_cs0 <= 0;
`else
	CS0 <= 0;
`endif
`ifdef FIXED_WGM0
`else
	WGM0[2:0] <= 0;
`endif
`ifdef FIXED_COM0B
`else
	COM0B <= 0;
`endif
`ifdef FIXED_COM0A
`else
	COM0A <= 0;
`endif
	tsm <= 0;
	psr <= 0;
`ifdef TIM0_HAVE_FOC
	foc0b <= 0;
	foc0a <= 0;
`endif
`ifdef IOR_HAVE_TIM0_OC0A
	r_oc0a <= 0;
`endif
`ifdef IOR_HAVE_TIM0_OC0B
	r_oc0b <= 0;
`endif

	r_tcnt0 <= 0;
	r_tim0_ovf   <= 0;
	r_tim0_compa <= 0;
	r_tim0_compb <= 0;
	t0_prev <= 0;
      end
    else
      begin
`ifdef FIXED_OCR0B
`else
	if(WE_ocr0b) r_ocr0b <= DI; 	 
`endif
`ifdef FIXED_OCR0A
`else
	if(WE_ocr0a) r_ocr0a <= DI;
`endif
	if(WE_tccr0a) 
	begin
`ifdef FIXED_COM0A
`else
	  COM0A <= DI[7:6];
`endif
`ifdef FIXED_COM0B
`else
	  COM0B <= DI[5:4];
`endif
`ifdef FIXED_WGM0
`else
	  WGM0[1:0] <= DI[1:0];
`endif
	end
	psr <= v_psr;

	if(WE_tccr0b) 
	begin
	  tsm <= DI[5];
`ifdef FIXED_WGM0
`else
	  WGM0[2] <= DI[3];
`endif
`ifdef FIXED_CS0
	  r_cs0 <= (DI[2:0] == 0)? 1'b0 : 1'b1; 
`else
	  CS0 <= DI[2:0];
`endif

`ifdef TIM0_HAVE_FOC
	  foc0a <= DI[7];
	  foc0b <= DI[6];
`endif

	end

	if(WE_tcnt0) r_tcnt0 <= DI;
	else begin
	  if (t0_nops | (t0_clk ^ t0_prev) & t0_prev)
	    if (t0_top_ocr0a & (r_tcnt0 == r_ocr0a)) r_tcnt0 <= 8'h00; 
	    else begin
		r_tcnt0 <= r_tcnt0 + 8'h01;
		r_tim0_ovf   <= (r_tcnt0 + 8'h01 == 8'hff);
		r_tim0_compa <= (r_tcnt0 + 8'h01 == r_ocr0a);
		r_tim0_compb <= (r_tcnt0 + 8'h01 == r_ocr0b);
	    end
	end
	t0_prev <= t0_clk;
`ifdef IOR_HAVE_TIM0_OC0A
	   begin
		if (COM0A == 1) 
		   if (r_tcnt0 + 8'h01 == r_ocr0a) r_oc0a <= ~r_oc0a;
		if (COM0A == 2) 
		   if (r_tcnt0 + 8'h01 == r_ocr0a) r_oc0a <= 0;
		   else if (r_tcnt0 + 8'h01 == 8'hff) r_oc0a <= 1;
		if (COM0A == 3) 
		   if (r_tcnt0 + 8'h01 == r_ocr0a) r_oc0a <= 1;
		   else if (r_tcnt0 + 8'h01 == 8'hff) r_oc0a <= 0;
	   end
`endif
`ifdef IOR_HAVE_TIM0_OC0B
	   begin
		if (COM0B == 1) 
		   if (r_tcnt0 + 8'h01 == r_ocr0b) r_oc0b <= ~r_oc0b;
		if (COM0B == 2) 
		   if (r_tcnt0 + 8'h01 == r_ocr0b) r_oc0b <= 0;
		   else if (r_tcnt0 + 8'h01 == 8'hff) r_oc0b <= 1;
		if (COM0B == 3) 
		   if (r_tcnt0 + 8'h01 == r_ocr0b) r_oc0b <= 1;
		   else if (r_tcnt0 + 8'h01 == 8'hff) r_oc0b <= 0;
	   end
`endif
      end
  end
endmodule

