/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

`include "rtavr_defs.v"

// global defines : 
//`define STALL_WB_CONFLICT_1
//  - STALL any WB inst + INDEX inst : not recomended
//`define STALL_WB_CONFLICT_2
//  - STALL LO8 WB inst + INDEX inst : need for gpr_16w (shrink mode)
//`define STALL_WB_CONFLICT_3
//  - STALL r26-r31 WB inst + INDEX X/Y/Z inst
//`define STALL_WB_CONFLICT_4
//  - STALL r26/r28/r30 WB inst + INDEX X/Y/Z inst  : gpr_16 only

//`define S0_INCLUDE_ROM

// Module Name: rtavr_s0_fetch
//   Ver 005
//
// Target Device:   xc3s200a-4ft256
// Product Version: ISE 12.4
// Design Goal:     Balanced
//
//                               NORMAL     w 8K ROM
// Number of Slice Flip Flops:        4         25  
// Number of 4 input LUTs:           26         36 
// Number of occupied Slices:        16         32    
// Total Number of 4 input LUTs:     26         36   
// Number of bonded IOBs :           54         81  
//   IOB Flip Flops :                16          -    
// Number of BUFGMUXs :               1          1   
// Number of RAMB16BWEs :             -          4  
// Clock to Setup 
//on destination clock CLK            ?      3.809|    

module rtavr_s0_fetch (
		input CLK,
		input RESET,
`ifdef S0_INCLUDE_ROM
		input [11:0] PC,
		input WEB, OE, 
		input [12:0] ADDRB,
		input [7:0] DIB, 
		output[7:0] DOB,
`else
		input [15:0] PM_OUT,
`endif
		input S0_VALID,
		output WB_INST,
		output IDX_INST,
		output [15:0] INST,
		output GPR_PREDEC, GPR_POSTINC,
		output [4:0] GPR_IDX_OFF,
`ifdef NEXTPC_TUNE
		output PD_EXT,
		output PD_JUMPS_R,
		output PD_JUMPS_I,
		output PD_SKIPS,
		output PD_BR,
  `ifdef EA_TUNE
		output PD_BSET_BCLR,
		output PD_EA_ABS,
		output PD_PUSH_POP,
		output PD_CALLS,
  `endif
`endif
		output[3:0] GPR_ADDRAL,
		output[3:0] GPR_ADDRAH,
		output[3:0] GPR_ADDRBL,
		output[3:0] GPR_ADDRBH
		);

  reg [15:0] s1_inst;
  reg s1_predec;
  reg s1_postinc;
  reg [3:0] s1_addr_ah;
  reg [3:0] s1_addr_bh;

  assign INST = s1_inst;
`ifdef S0_INCLUDE_ROM
  wire [15:0] PM_OUT;
  rtavr_rom #(.SIZE(4096)) PM(.CLK(CLK) , .ADDRA(PC), .DOA(PM_OUT) , 
		 .WEB(WEB) , .OE(OE) , .ADDRB(ADDRB),
		 .DIB(DIB), .DOB(DOB) 
		);
`endif

`ifdef NEXTPC_TUNE
  reg pd0_ext;
//  reg pd0_ij;
//  reg pd0_ic;
//  reg pd0_rj;
//  reg pd0_rc;
  reg pd0_jumps_i;
  reg pd0_jumps_r;
  reg pd0_br;
//  reg pd0_sbix;
//  reg pd0_sbrx;
//  reg pd0_cpse;
  reg pd0_skips;

    assign PD_EXT     = pd0_ext;
    assign PD_JUMPS_I = pd0_jumps_i;
    assign PD_JUMPS_R = pd0_jumps_r;
    assign PD_SKIPS   = pd0_skips;
    assign PD_BR      = pd0_br;

  wire f_ext = (PM_OUT[15:9] == 7'b1001010 );

// IJMP            1001:0100:0000:1001     # 
// ICALL           1001:0101:0000:1001     # 
//                 EEEE:EEE*:----:1--1     # 

  wire f_ij =  f_ext & (~PM_OUT[8] & PM_OUT[3] & PM_OUT[0]);
  wire f_ic =  f_ext & (PM_OUT[8] & PM_OUT[3] & PM_OUT[0]);

// RJMP            1100:kkkk:kkkk:kkkk     # 
// RCALL           1101:kkkk:kkkk:kkkk     # 

  wire [11:0] adr_rj = PM_OUT[11:0];
  wire f_rj =  (PM_OUT[15:12] == 4'b1100 );
  wire f_rc =  (PM_OUT[15:12] == 4'b1101 );

  wire f_jumps = ( f_ij | f_ic | f_rj | f_rc );

// BRBC            1111:01kk:kkkk:ksss     # 
// BRBS            1111:00kk:kkkk:ksss     #

  wire [11:0] adr_br = { PM_OUT[9], PM_OUT[9], PM_OUT[9], PM_OUT[9], PM_OUT[9]
			 ,PM_OUT[9:3] };

  wire f_br = (PM_OUT[15:11] == 5'b11110 );


// SBIC            1001:1001:AAAA:Abbb     #
// SBIS            1001:1011:AAAA:Abbb     #

// SBRC            1111:110r:rrrr:0bbb     #
// SBRS            1111:111r:rrrr:0bbb     #

// CPSE		   000B:00rd:dddd:rrrr	# B=1: CPSE           #   ------

  wire f_sbix = (PM_OUT[15:10] == 6'b100110) & PM_OUT[8];
  wire f_sbrx = (PM_OUT[15:10] == 6'b111111 );
  
  wire f_cpse = (PM_OUT[15:10] == 6'b000100);
  wire f_skips = ( f_cpse | f_sbrx | f_sbix );

  `ifdef EA_TUNE
     reg pd0_bset_bclr;
     reg pd0_ea_abs;
     reg pd0_push_pop;
     reg pd0_calls;

     assign PD_BSET_BCLR = pd0_bset_bclr;
     assign PD_EA_ABS    = pd0_ea_abs;
     assign PD_PUSH_POP  = pd0_push_pop;
     assign PD_CALLS     = pd0_calls;

     wire f_bset_bclr = f_ext & ~PM_OUT[8] & (PM_OUT[3:0] == 4'b1000) ;

     wire f_ea_abs = (PM_OUT[15:13] == 3'b101)
		   | (PM_OUT[15:10] == 6'b100110)
		   | f_bset_bclr ;

     wire f_push_pop = (PM_OUT[15:10] == 6'b100100)
			& (PM_OUT[3:0] == 4'b1111);
  `endif
`endif
//         LD.Rd.--X	1001:000d:dddd:1110
//         ST.--X.Rr	1001:001r:rrrr:1110
//         LD.Rd.--Y	1001:000d:dddd:1010
//         ST.--Y.Rr	1001:001r:rrrr:1010
//         LD.Rd.--Z	1001:000d:dddd:0010
//         ST.--Z.Rr	1001:001r:rrrr:0010
//         ELPM.Rd.Z    1001:000d:dddd:0110  << undefined instruction
  wire v_predec = ( PM_OUT[15:10] == 6'b100100 ) & ( PM_OUT[1:0] == 2'b10 );
//         LD.Rd.X++	1001:000d:dddd:1101
//         ST.X++.Rr	1001:001r:rrrr:1101
//         LD.Rd.Y++	1001:000d:dddd:1001
//         ST.Y++.Rr	1001:001r:rrrr:1001
//         LD.Rd.Z++	1001:000d:dddd:0001
//         ST.Z++.Rr	1001:001r:rrrr:0001
//         LPM.Rd.Z++   1001:000d:dddd:0101  << undefined instruction
  wire v_postinc = ( PM_OUT[15:10] == 6'b100100 ) & ( PM_OUT[1:0] == 2'b01 );
//       0xa,0xb: (26 - 16, 27 - 16)
//         LD.Rd.X	1001:000d:dddd:1100
//         ST.X.Rr	1001:001r:rrrr:1100
//         LD.Rd.X++	1001:000d:dddd:1101
//         ST.X++.Rr	1001:001r:rrrr:1101
//         LD.Rd.--X	1001:000d:dddd:1110
//         ST.--X.Rr	1001:001r:rrrr:1110
//         POP          1001:000d:dddd:1111  <<<
//         PUSH         1001:001d:dddd:1111  <<<
  wire v_xreg = ( PM_OUT[15:10] == 6'b100100 ) & ( PM_OUT[3:2] == 2'b11 )
                  & ( PM_OUT[1:0] != 2'b11 );
//       0xc,0xd:(28 - 16, 29 - 16)
//         LD.Rd.Y	1000:000d:dddd:1000
//         ST.Y.Rr	1000:001r:rrrr:1000
//         LD.Rd.Y++	1001:000d:dddd:1001
//         ST.Y++.Rr	1001:001r:rrrr:1001
//         LD.Rd.--Y	1001:000d:dddd:1010
//         ST.--Y.Rr	1001:001r:rrrr:1010
//      LDD.Rd.Y+q      1000:qq0d:dddd:1qqq     # (31 >= q >= 0) : reduced ver
//      STD.Y+q.Rr      1000:qq1r:rrrr:1qqq     # (31 >= q >= 0) : reduced ver

  wire v_yreg = ( PM_OUT[15:10] == 6'b100100 ) & ( PM_OUT[3:2] == 2'b10 )
`ifdef SUPPORT_LDD
		| ( PM_OUT[15:12] == 4'b1000 ) & PM_OUT[3]
`endif
		| ( PM_OUT[15:12] == 4'b1000 ) & ( PM_OUT[3:2] == 2'b10 );
//       0xe,0xf:(30 - 16, 31 - 16)
//         LD.Rd.Z	1000:000d:dddd:0000
//         ST.Z.Rr	1000:001r:rrrr:0000
//         LD.Rd.Z++	1001:000d:dddd:0001
//         ST.Z++.Rr	1001:001r:rrrr:0001
//         LD.Rd.--Z	1001:000d:dddd:0010
//         ST.--Z.Rr	1001:001r:rrrr:0010
//      LDD.Rd.Z+q      1000:qq0d:dddd:0qqq     # (31 >= q >= 0) : reduced ver
//      STD.Z+q.Rr      1000:qq1r:rrrr:0qqq     # (31 >= q >= 0) : reduced ver
  wire v_zreg = ( PM_OUT[15:10] == 6'b100100 ) & ( PM_OUT[3:2] == 2'b00 )
`ifdef SUPPORT_LDD
		| ( PM_OUT[15:12] == 4'b1000 ) & ~PM_OUT[3]
`endif
		| ( PM_OUT[15:12] == 4'b1000 ) & ( PM_OUT[3:2] == 2'b00 );
//        0xe,0xf:(30 - 16, 31 - 16)
//          IJMP	1001:0100:0000:1001
//          ICALL	1001:0101:0000:1001
//          EIJMP       1001:0100:0001:1001  << undefined instruction
//          EICALL      1001:0101:0001:1001  << undefined instruction
  wire v_icall = ( PM_OUT[15:10] == 6'b100101 ) & PM_OUT[3] & PM_OUT[0];

// LOAD Rr (Rd -> Rr @s0)
// LD.Rd.Z         1000:000d:dddd:0000     # 
// LD.Rd.Y         1000:000d:dddd:1000     # 
// LD.Rd.Z++       1001:000d:dddd:0001     # 
// LD.Rd.--Z       1001:000d:dddd:0010     # 
// LD.Rd.Y++       1001:000d:dddd:1001     # 
// LD.Rd.--Y       1001:000d:dddd:1010     # 
// LD.Rd.X         1001:000d:dddd:1100     # 
// LD.Rd.X++       1001:000d:dddd:1101     # 
// LD.Rd.--X       1001:000d:dddd:1110     # 

// POP             1001:000d:dddd:1111     # 

// SYORE Rr
// ST.Z.Rr         1000:001r:rrrr:0000     # 
// ST.Y.Rr         1000:001r:rrrr:1000     # 
// ST.Z++.Rr       1001:001r:rrrr:0001     # 
// ST.--Z.Rr       1001:001r:rrrr:0010     # 
// ST.Y++.Rr       1001:001r:rrrr:1001     # 
// ST.--Y.Rr       1001:001r:rrrr:1010     # 
// ST.X.Rr         1001:001r:rrrr:1100     # 
// ST.X++.Rr       1001:001r:rrrr:1101     # 
// ST.--X.Rr       1001:001r:rrrr:1110     # 

// PUSH            1001:001d:dddd:1111     # 
//                 100X:00S-----------

// LPM.Rd.Z        1001:000d:dddd:0100     # undefined inst
// LPM.Rd.Z++      1001:000d:dddd:0101     # undefined inst
// ELPM.Rd.Z       1001:000d:dddd:0110     # undefined inst
// ELPM.Rd.Z++     1001:000d:dddd:0111     # undefined inst
// LDS(2word)      1001:000d:dddd:0000     # undefined inst
// STS(2word)      1001:001d:dddd:0000     # undefined inst

  // LD/ST + POP/PUSH
  `ifdef SUPPORT_LDD
     wire v_ldst = (PM_OUT[15:13] == 3'b100) & (PM_OUT[11:10] == 2'b00)
  			| (PM_OUT[15:12] == 4'b1000);
  `else
     wire v_ldst = (PM_OUT[15:13] == 3'b100) & (PM_OUT[11:10] == 2'b00);
  `endif
  
//      LDD.Rd.Z+q      1000:qq0d:dddd:0qqq     # (31 >= q >= 0) : reduced ver
//      STD.Z+q.Rr      1000:qq1r:rrrr:0qqq     # (31 >= q >= 0) : reduced ver
`ifdef SUPPORT_LDD
  reg [4:0] s_idx_off;
  wire v_lddstd = PM_OUT[15:12] == 4'b1000 ;
  wire [4:0] v_idx_off = v_lddstd ? { PM_OUT[11:10] , PM_OUT[2:0] } : 0;

  assign GPR_IDX_OFF  = s_idx_off;
`else
  assign GPR_IDX_OFF  = 0;
`endif

wire [3:0] v_addr_ah    = INST[7:4];
assign GPR_ADDRAH   = s1_addr_ah;

assign WB_INST   =   v_predec | v_postinc;
`ifdef STALL_WB_CONFLICT_1
      assign IDX_INST  =   v_xreg | v_yreg | v_zreg;
`elsif STALL_WB_CONFLICT_2
      assign IDX_INST  =   ( v_xreg | v_yreg | v_zreg ) & ~v_addr_ah[0];
`elsif STALL_WB_CONFLICT_3
      assign IDX_INST  =    v_xreg & ( v_addr_ah == 4'hA )  // (r26)
			  | v_xreg & ( v_addr_ah == 4'hB )  // (r27)
			  | v_yreg & ( v_addr_ah == 4'hC )  // (r28)
			  | v_yreg & ( v_addr_ah == 4'hD )  // (r29)
			  | v_zreg & ( v_addr_ah == 4'hE )  // (r30)
			  | v_zreg & ( v_addr_ah == 4'hF ); // (r31)
`elsif STALL_WB_CONFLICT_4
      assign IDX_INST  =    v_xreg & ( v_addr_ah == 4'hA )  // (r26)
			  | v_yreg & ( v_addr_ah == 4'hC )  // (r28)
			  | v_zreg & ( v_addr_ah == 4'hE ); // (r30)
`else
      assign IDX_INST  =   0;
`endif
assign GPR_PREDEC  = s1_predec;
assign GPR_POSTINC = s1_postinc;
assign GPR_ADDRAL   = v_xreg ? 4'hA : v_yreg ? 4'hC    // (r26) (r28)
                    : v_zreg ? 4'hE : PM_OUT[7:4];     // (r30)

assign GPR_ADDRBL   = v_icall ? 4'hE : PM_OUT[7:4];       // (r30)

wire [3:0] v_addr_bh  = (v_icall | v_ldst & v_zreg) ? 4'hF  // (r31)
                        : (v_ldst & v_yreg) ? 4'hD          // (r29)
                        : (v_ldst & v_xreg) ? 4'hB          // (r27)
		        : PM_OUT[3:0];
assign GPR_ADDRBH   = s1_addr_bh;

  always @(negedge CLK)
  begin
    if (RESET == 0)
      begin
	s1_inst    <= 0;
	s1_predec  <= 0;
	s1_postinc <= 0;
	s1_addr_ah <= 0;
	s1_addr_bh <= 0;
`ifdef SUPPORT_LDD
	s_idx_off  <= 0;
`endif
`ifdef NEXTPC_TUNE
	pd0_ext     <=0;
	pd0_jumps_i <=0;
	pd0_jumps_r <=0;
	pd0_skips   <=0;
	pd0_br      <=0;
  `ifdef EA_TUNE
	pd0_bset_bclr <= 0;
	pd0_ea_abs    <= 0;
	pd0_push_pop  <= 0;
	pd0_calls     <= 0;
  `endif
`endif
      end
    else
      begin
	s1_inst <= PM_OUT;
	s1_predec  <= S0_VALID & v_predec;
	s1_postinc <= S0_VALID & v_postinc;
	s1_addr_ah <= v_addr_ah;
	s1_addr_bh <= v_addr_bh;
`ifdef SUPPORT_LDD
	s_idx_off  <= v_idx_off;
`endif
`ifdef NEXTPC_TUNE
	pd0_ext     <= f_ext;
	pd0_jumps_i <= f_ij | f_ic ;
	pd0_jumps_r <= f_rj | f_rc ;
	pd0_skips   <= f_skips;
	pd0_br      <= f_br;
  `ifdef EA_TUNE
	pd0_bset_bclr <= f_bset_bclr;
	pd0_ea_abs    <= f_ea_abs;
	pd0_push_pop  <= f_push_pop;
	pd0_calls     <= (f_rc | f_ic);
  `endif
`endif
      end
  end

endmodule
