/*
 * Copyright (C) 2011 Koji Suzuki
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2.1 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

`include "rtavr_defs.v"

// global defines : OUTPUT_MUX_TRI

//`define OUTPUT_MUX_NONE
//`define OUTPUT_MUX_TRI

// Module Name: rtavr_ram_2KB
//  ver 005
// 
// Target Device:   xc3s200a-4ft256
// Product Version: ISE 12.4
// Design Goal:     Balanced
//
//                                NORMAL  MUX_TRI(FAKE) MUX_OR      
// Number of Slice Flip Flops:      8          0          0   
// Number of 4 input LUTs:          4          0          0
// Number of occupied Slices:       4          0          0
// Total Number of 4 input LUTs:    8          0          0
// Number of bonded IOBs :         58         58         57
//   IOB Flip Flops :               -          -          -
// Number of BUFGMUXs :             1          1          1
// Number of RAMB16BWEs :           1          1          1
// Clock to Setup 
//on destination clock CLK          ?          ?          ?


//        ___        ______        ______
//  CLK2X    |______|      |______|      |______
//
//        _____               ______________
//  CLK        |_____________|              |___
//
// Normal                    |
//                         read/write
// CLK2X
//                   |            |
//                 read         write

module rtavr_sram_2KB(
		input CLK2X,
		input CLK,
`ifdef RAM_HAVE_PORTA
		input WEA,
		input [10:0] ADDRA,
		input [7:0] DIA,
		output [7:0] DOA,
`endif
		input WEB,
`ifdef OUTPUT_MUX_NONE
`else
		input OE,
`endif
		input [10:0] ADDRB,
		input [7:0] DIB,
		output [7:0] DOB
		);

`ifdef RAM_CLK_LOGIC
  wire CLK_B = CLK2X & ~(WEB ^ CLK);
  `ifdef RAM_HAVE_PORTA 
      wire CLK_A = CLK2X & ~(WEA ^ CLK);
  `endif
`elsif RAM_CLK_LATCH
  reg CLK_B;
  always @(CLK2X or CLK or WEB)
  begin
	if (~CLK2X) CLK_B <= 0;
	else if (CLK2X & ~(WEB ^ CLK)) CLK_B <= 1;
  end
  `ifdef RAM_HAVE_PORTA 
      always @(CLK2X or CLK or WEA)
      begin
	if (~CLK2X) CLK_A <= 0;
	else if (CLK2X & ~(WEA ^ CLK)) CLK_A <= 1;
      end
  `endif
`else
  wire CLK_B = CLK2X;
  `ifdef RAM_HAVE_PORTA 
      wire CLK_A = CLK2X;
  `endif
  `define RAM_DRIVEN_CLK2X
`endif


  reg  [7:0] mem [0:2048-1];

`ifdef RAM_HAVE_PORTA
  reg  [10:0] r_addra;
 
  always @(posedge CLK_A)
  begin
  `ifdef RAM_DRIVEN_CLK2X
	if(WEA & CLK) mem[ADDRA] <= DIA;
	r_addra <= ADDRA;
  `else
	if(WEA) mem[ADDRA] <= DIA;
	r_addra <= ADDRA;
  `endif
  end
 assign DOA = mem[r_addra];
`endif
 
  reg  [10:0] r_addrb;

  always @(posedge CLK_B)
  begin
  `ifdef RAM_DRIVEN_CLK2X
	if(WEB & CLK) mem[ADDRB] <= DIB;
	r_addrb <= ADDRB;
  `else
	if(WEB) mem[ADDRB] <= DIB;
	r_addrb <= ADDRB;
  `endif
  end
`ifdef OUTPUT_MUX_NONE
  assign DOB = mem[r_addrb];
`elsif OUTPUT_MUX_TRI
  bufif1(DOB, mem[r_addrb], OE);
`else
  assign DOB = OE? mem[r_addrb] : 8'h00;
`endif

`ifdef SRAM_INITCLR
  integer i;
  initial
    begin
      for (i = 0 ; i < 2048 ; i = i + 1)
	mem[i] <= 8'h00;
    end
`endif
endmodule
